require 'test_helper'

context 'class methods without setup' do
  teardown do
    AllSeeingEye.redis.flushall
  end
  
  test 'models know their own name' do
    assert_equal 'request', AllSeeingEye::Request.model_name
  end
  
  test 'models correctly find their next ID' do
    id = AllSeeingEye::Request.next_id
    (id + 1..(id + 11)).each {|n|assert_equal AllSeeingEye::Request.next_id, n}
  end
  
  test 'models should get fields' do
    assert_equal AllSeeingEye::Request.fields,
      {"uri"=>{"method"=>".request_uri", "object"=>"request"}, "action"=>{"method"=>"[:action]", "object"=>"params"}, "ip"=>{"method"=>".remote_ip", "object"=>"request"}, "controller"=>{"method"=>"[:controller]", "object"=>"params"}, "browser"=>{"method"=>".env[\"HTTP_USER_AGENT\"]", "object"=>"request"}, "status"=>{"method"=>".status", "object"=>"response"}}
  end
  
  test 'models should get field names' do
    assert_equal ["uri", "action", "ip", "controller", "browser", "status", "time_spent", "created_at"],
      AllSeeingEye::Request.field_names
  end
  
  test 'return a new model object' do
    time = Time.now
    obj = AllSeeingEye::Request.create(:uri => '/test/', :ip => '127.0.0.1', :created_at => time)
    
    assert_equal '/test/', obj.uri
    assert_equal '127.0.0.1', obj.ip
    assert_equal time, obj.created_at
    assert_equal nil, obj.action
  end
  
  test 'create a model object' do
    obj = AllSeeingEye::Request.create(:uri => '/test/', :ip => '127.0.0.1')
    
    assert_equal '/test/', obj.uri
    assert_equal '127.0.0.1', obj.ip
    assert_equal Marshal.dump(obj), AllSeeingEye.redis["allseeingeye:request:#{AllSeeingEye::Request.next_id - 1}"]
    assert_equal '1', AllSeeingEye.redis.zscore("allseeingeye:request:fields:uri", '/test/')
    assert_equal '1', AllSeeingEye.redis.zscore("allseeingeye:request:fields:ip", '127.0.0.1')
    assert_equal [obj.id.to_s], AllSeeingEye.redis.sort("allseeingeye:request:fields:uri:/test/", :by => 'nosort')
    assert_equal [obj.id.to_s], AllSeeingEye.redis.sort("allseeingeye:request:fields:ip:127.0.0.1", :by => 'nosort')
  end
  
  test 'unmarshal an object with load' do
    time = Time.now
    raw = AllSeeingEye::Request.create(:uri => '/test/', :ip => '127.0.0.1', :created_at => time)
    dump = Marshal.dump(raw)
    obj = AllSeeingEye::Request.load(dump)
    
    assert_equal '/test/', obj.uri
    assert_equal '127.0.0.1', obj.ip
    assert_equal time, obj.created_at
    assert_equal raw.id, obj.id
  end
  
  test 'find an object by its ID' do
    time = Time.now
    obj = AllSeeingEye::Request.create(:uri => '/test/', :ip => '127.0.0.1', :created_at => time)
    found = AllSeeingEye::Request.find(obj.id)
    
    [:uri, :ip, :created_at].each {|f| assert_equal obj.send(f), found.send(f)}
  end
  
  test 'find all objects' do
    obj1 = AllSeeingEye::Request.create(:uri => '/test/', :ip => '127.0.0.1', :created_at => Time.now)
    obj2 = AllSeeingEye::Request.create(:uri => '/test/', :ip => '127.0.0.1', :created_at => Time.now)
    obj3 = AllSeeingEye::Request.create(:uri => '/test/', :ip => '127.0.0.1', :created_at => Time.now)
    
    assert_equal [obj1.id, obj2.id, obj3.id], AllSeeingEye::Request.all.collect(&:id)
  end
  
  test 'find the count of all objects' do
    obj1 = AllSeeingEye::Request.create(:uri => '/test/', :ip => '127.0.0.1', :created_at => Time.now)
    obj2 = AllSeeingEye::Request.create(:uri => '/test/', :ip => '127.0.0.1', :created_at => Time.now)
    obj3 = AllSeeingEye::Request.create(:uri => '/test/', :ip => '127.0.0.1', :created_at => Time.now)
    
    assert_equal 3, AllSeeingEye::Request.count
  end
end

context 'class methods with setup' do
  setup do
    @time = Time.now
    
    @obj1 = AllSeeingEye::Request.create(:uri => '/', :ip => '127.0.0.1', :created_at => @time - 10 * 60)
    @obj2 = AllSeeingEye::Request.create(:uri => '/test/', :ip => '127.0.0.1', :created_at => @time - 5 * 60)
    @obj3 = AllSeeingEye::Request.create(:uri => '/test/', :ip => '127.0.0.1', :created_at => @time - 3 * 60)
    @obj4 = AllSeeingEye::Request.create(:uri => '/test/', :ip => '192.168.0.1', :created_at => @time - 1 * 60)
    @obj5 = AllSeeingEye::Request.create(:uri => '/test/', :ip => '127.0.0.1', :created_at => @time + 2 * 60)
    @obj6 = AllSeeingEye::Request.create(:uri => '/test/', :ip => '192.168.0.1', :created_at => @time + 2 * 60)
  end
  
  teardown do
    AllSeeingEye.redis.flushall
  end
  
  test 'find all objects in a date range with a start and a stop' do
    results = AllSeeingEye::Request.all(:start => @time - 11 * 60, :stop => @time - 4 * 60)
    
    assert_equal 2, results.count
    assert_equal @obj1.id, results[0].id
    assert_equal @obj2.id, results[1].id
  end
  
  test 'find all objects in a date range with only a start' do
    results = AllSeeingEye::Request.all(:start => @time)
    
    assert_equal 2, results.count
    assert_equal @obj5.id, results[0].id
    assert_equal @obj6.id, results[1].id
  end
  
  test 'find all objects in a date range with only a stop' do
    results = AllSeeingEye::Request.all(:stop => @time)
    
    assert_equal 4, results.count
    assert_equal @obj1.id, results[0].id
    assert_equal @obj2.id, results[1].id   
    assert_equal @obj3.id, results[2].id
    assert_equal @obj4.id, results[3].id   
  end
  
  test 'find model objects by a string field' do
    results = AllSeeingEye::Request.find_by_field('uri', :value => '/test/')

    assert_equal 5, results.count
    assert results.collect(&:id).include?(@obj2.id)
    assert results.collect(&:id).include?(@obj3.id)
    assert results.collect(&:id).include?(@obj4.id)
    assert results.collect(&:id).include?(@obj5.id)
    assert results.collect(&:id).include?(@obj6.id)
  end
  
  test 'find model objects by a string field with a start' do
    results = AllSeeingEye::Request.find_by_field('uri', :value => '/test/', :start => @time - 4 * 60)
    
    assert_equal 4, results.count
    assert results.collect(&:id).include?(@obj3.id)
    assert results.collect(&:id).include?(@obj4.id)
    assert results.collect(&:id).include?(@obj5.id)
    assert results.collect(&:id).include?(@obj6.id)
  end
  
  test 'find model objects by a string field with a stop' do
    results = AllSeeingEye::Request.find_by_field('uri', :value => '/test/', :stop => @time)
    
    assert_equal 3, results.count
    assert results.collect(&:id).include?(@obj2.id)
    assert results.collect(&:id).include?(@obj3.id)
    assert results.collect(&:id).include?(@obj4.id)
  end
  
  test 'find model objects by a string field with a start and a stop' do
    results = AllSeeingEye::Request.find_by_field('uri', :value => '/test/', :start => @time - 4 * 60, :stop => @time)
    
    assert_equal 2, results.count
    assert results.collect(&:id).include?(@obj3.id)
    assert results.collect(&:id).include?(@obj4.id)
  end
  
  test 'find model objects with a limit and offset' do
    
  end
  
  test 'find model objects by a date field' do
    results = AllSeeingEye::Request.find_by_field('created_at', :start => @obj1.created_at, :stop => @obj4.created_at)
    
    assert_equal 4, results.count
    assert_equal results.first.id, @obj1.id
    assert_equal results.last.id, @obj4.id
  end
  
  test 'count model objects' do
    assert_equal 6, AllSeeingEye::Request.count
  end
  
  test 'return the first object' do
    assert_equal @obj1.id, AllSeeingEye::Request.first.id
  end
  
  test 'return the last object' do
    assert_equal @obj6.id, AllSeeingEye::Request.last.id
  end
  
  test 'return count in a field' do
    counts = AllSeeingEye::Request.list_by_field('uri')

    assert_equal '/test/', counts.first.first
    assert_equal 5, counts.first.last
    assert_equal '/', counts.last.first
    assert_equal 1, counts.last.last
  end
  
  test 'search with Chronic' do
    results = AllSeeingEye::Request.search('last friday to five minutes ago')
    
    assert_equal 2, results.size
    assert_equal @obj1.id, results[0].id
    assert_equal @obj2.id, results[1].id
  end
  
  test 'search by a specific field' do
    results = AllSeeingEye::Request.search('uri:/test/')
    
    assert_equal 5, results.count
    assert results.collect(&:id).include?(@obj2.id)
    assert results.collect(&:id).include?(@obj3.id)
    assert results.collect(&:id).include?(@obj4.id)
    assert results.collect(&:id).include?(@obj5.id)
    assert results.collect(&:id).include?(@obj6.id)
  end
  
  test 'search for a count on a specific field' do
    results = AllSeeingEye::Request.search('uri:/test/', :count => true)
    
    assert_equal 4, results.count
    assert_equal results.last, [((@obj6.created_at.to_i / 60) * 60).to_s, 2]
  end
end

context 'many objects' do
  setup do
    AllSeeingEye.redis.flushall
    @time = Time.now
    
    2000.times do |n|
      instance_variable_set("@obj#{n}".to_sym, AllSeeingEye::Request.create(:uri => '/', :ip => n % 2 == 0 ? '127.0.0.1' : '192.168.0.1', :created_at => @time - (n * (1999 - n)) * 60))
    end
  end
    
  test 'get the created count' do
    counts = AllSeeingEye::Request.list_by_field('created_at')
    
    assert_equal counts, AllSeeingEye::Request.search('ten years ago to one year from now', :count => true)
    assert_equal (@obj999.created_at.to_i / 60) * 60, counts.first.first.to_i
    assert_equal (@obj0.created_at.to_i / 60) * 60, counts.last.first.to_i
    assert_equal 2000, counts.inject(0) {|sum, k| sum + k.last}
  end
  
  test 'get the created count for a value' do
    counts = AllSeeingEye::Request.list_by_field('ip', :value => '192.168.0.1')
    
    assert_equal 1000, counts.inject(0) {|sum, k| sum + k.last}
  end
  
  test 'get only the first page of the count' do
    counts = AllSeeingEye::Request.list_by_field('created_at', :page => 1)
    
    assert_equal 10, counts.size
    assert_equal (@obj999.created_at.to_i / 60) * 60, counts.first.first.to_i
    assert_equal (@obj990.created_at.to_i / 60) * 60, counts.last.first.to_i
  end
  
  test 'get only the second page of the count' do
    counts = AllSeeingEye::Request.list_by_field('created_at', :page => 2)
    
    assert_equal 10, counts.size
    assert_equal (@obj989.created_at.to_i / 60) * 60, counts.first.first.to_i
    assert_equal (@obj980.created_at.to_i / 60) * 60, counts.last.first.to_i
  end
  
  test 'get only the first page of the count with a value' do
    counts = AllSeeingEye::Request.list_by_field('ip', :value => '192.168.0.1', :page => 1)
    
    assert_equal 10, counts.size
  end
  
  test 'return the count for a field with a given value' do
    assert_equal 1000, AllSeeingEye::Request.count_by_field('ip', :value => '192.168.0.1')
  end
end

context 'instance methods' do  
  teardown do
    AllSeeingEye.redis.flushall
  end
  
  test 'assign all attr_accessors from fields at initialization' do
    obj = AllSeeingEye::Request.new
    
    AllSeeingEye::Request.field_names.each {|f| assert obj.respond_to?(f.to_sym); assert obj.respond_to?("#{f}=".to_sym)}
  end
  
  test 'assign an object an ID if it does not have one' do
    obj = AllSeeingEye::Request.new
    
    assert_equal nil, obj.instance_variable_get(:@id)
    assert_equal 1, obj.id
    assert_equal 1, obj.instance_variable_get(:@id)
  end
  
  test 'save a new model object' do
    obj = AllSeeingEye::Request.new(:uri => '/test/', :ip => '127.0.0.1')
    
    assert obj.save
    assert_equal '/test/', obj.uri
    assert_equal '127.0.0.1', obj.ip
    assert_equal Marshal.dump(obj), AllSeeingEye.redis["allseeingeye:request:#{AllSeeingEye::Request.next_id - 1}"]
    assert_equal '1', AllSeeingEye.redis.zscore("allseeingeye:request:fields:uri", '/test/')
    assert_equal '1', AllSeeingEye.redis.zscore("allseeingeye:request:fields:ip", '127.0.0.1')
    assert_equal [obj.id.to_s], AllSeeingEye.redis.sort("allseeingeye:request:fields:uri:/test/", :by => 'nosort')
  end
  
  test 'save a new model object with date attributes' do
    time = Time.now
    obj = AllSeeingEye::Request.new(:created_at => time)
    
    assert obj.save
    assert_equal time, obj.created_at
    assert_equal Marshal.dump(obj), AllSeeingEye.redis["allseeingeye:request:#{AllSeeingEye::Request.next_id - 1}"]
    assert_equal time.to_i.to_s, AllSeeingEye.redis.zscore("allseeingeye:request:fields:created_at", obj.id)
  end
  
  test 'ignore empty and nil attributes in model object creation' do
    time = Time.now
    obj = AllSeeingEye::Request.new(:created_at => time, :uri => '/test/', :ip => nil, :action => '')
    
    assert obj.save
    assert_equal time, obj.created_at
    assert_equal '/test/', obj.uri
    assert_equal nil, obj.ip
    assert_equal '', obj.action
    assert_equal Marshal.dump(obj), AllSeeingEye.redis["allseeingeye:request:#{AllSeeingEye::Request.next_id - 1}"]
    assert_equal time.to_i.to_s, AllSeeingEye.redis.zscore("allseeingeye:request:fields:created_at", obj.id)
    assert_equal '1', AllSeeingEye.redis.zscore("allseeingeye:request:fields:uri", '/test/')
    assert_equal nil, AllSeeingEye.redis.zscore("allseeingeye:request:fields:ip", nil)
    assert_equal nil, AllSeeingEye.redis.zscore("allseeingeye:request:fields:action", '')
    assert_equal [obj.id.to_s], AllSeeingEye.redis.sort("allseeingeye:request:fields:uri:/test/", :by => 'nosort')
    assert_equal [], AllSeeingEye.redis.sort("allseeingeye:request:fields:ip:", :by => 'nosort')
    assert_equal [], AllSeeingEye.redis.sort("allseeingeye:request:fields:action:", :by => 'nosort')
  end
  
end